/*****************************************************************************
 *
 * FILE:	basilisk.h
 * DESCRIPTION:	Basilisk: simple HTML authoring library header
 * DATE:	Sun, Mar 31 2002
 * UPDATE:	Sun, Aug 19 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2002-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2002-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: basilisk.h,v 1.4 2007/08/19 16:48:36 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_BASILISK_H
#define	_BASILISK_H

/*****************************************************************************/

/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	BASILISK_VERSION_NUMBER		0x10000020L
#define	BASILISK_VERSION		"Basilisk/1.0"
#define	BASILISK_VERSION_TEXT		BASILISK_VERSION " (2007/08/19)"
#define	BASILISK_VERSION_TEXT_LONG \
	"Basilisk HTML Library 1.0, Sun, Aug 19 2007"

/*****************************************************************************/

/* useful macros */
#ifndef	EOL
#define	EOL	'\0'
#endif	/* EOL */
#ifndef	CR
#define	CR	'\015'
#endif	/* CR */
#ifndef	LF
#define	LF	'\012'
#endif	/* LF */
#ifndef	CRLF
#define	CRLF	"\015\012"
#endif	/* CRLF */
#ifndef	SPC
#define	SPC	'\040'
#endif	/* SPC */
#ifndef	TAB
#define	TAB	'\011'
#endif	/* TAB */


#ifndef	_STRING_T
#define	_STRING_T	1
typedef char *	String;
#endif	/* _STRING_T */

#ifndef	_BOOL_T
#define	_BOOL_T	1
typedef enum {	/* boolean type */
  false = 0,
  true	= 1
} bool;
#endif	/* _BOOL_T */

/*****************************************************************************/

typedef char *		Length;
typedef unsigned int	Pixels;

typedef enum {	/* HTML3.2(headings,p,div) */
  HALIGN_NONE,
  HALIGN_LEFT,
  HALIGN_CENTER,
  HALIGN_RIGHT
} halign_t;

typedef enum {	/* HTML2.0(img, tr) */
  VALIGN_NONE,
  VALIGN_TOP,
  VALIGN_MIDDLE,
  VALIGN_BOTTOM
} valign_t;

typedef enum {	/* HTML3.2(caption) */
  CALIGN_NONE,
  CALIGN_TOP,
  CALIGN_BOTTOM
} calign_t;

typedef enum {	/* HTML3.2(img, input.image) */
  ALIGN_NONE,
  ALIGN_TOP,
  ALIGN_MIDDLE,
  ALIGN_BOTTOM,
  ALIGN_LEFT,
  ALIGN_RIGHT
} align_t;

typedef enum {	/* HTML3.2(ul) */
  BULLET_NONE,	/* default */
  BULLET_DISC,
  BULLET_SQUARE,
  BULLET_CIRCLE
} bullet_t;

typedef enum {	/* HTML3.2(ol) */
  NUMBERING_NONE,
  NUMBERING_ARABIC_NUMBER,
  NUMBERING_LOWER_ALPHA,
  NUMBERING_UPPER_ALPHA,
  NUMBERING_LOWER_ROMAN,
  NUMBERING_UPPER_ROMAN
} numbering_t;

typedef enum {	/* HTML document types */
  HTML_NONE,
  HTML2,
  HTML3,
  HTML4,
  HTML4Strict,
  HTML4Frame,
  HTML4Trans
} html_t;

typedef enum {
  METHOD_GET,
  METHOD_POST
} method_t;

typedef struct _Attrib {
  union {
    struct {
      String	bgcolor;	/* background color */
      String	text;		/* text color */
      String	link;		/* unvisited links color */
      String	vlink;		/* visited links color */
      String	alink;		/* highlight color */
      String	background;	/* image URL for document */
    } body;
    struct {
      bullet_t	bullet;		/* bullet style */
      bool	compact;	/* reduced interitem spacing */
    } ul;
    struct {
      numbering_t	numbering;	/* numbering style */
      unsigned int	start;		/* starting sequence */
      bool		compact;	/* reduced interitem spacing */
    } ol;
    struct {
      halign_t	halign;
      Pixels	size;
      Length	width;
      bool	noshade;
    } hr;
    struct {
      Pixels	border;		/* controls frame width around table */
      Length	width;		/* table width relative to window */
      Pixels	cellspacing;	/* spacing between cells */
      Pixels	cellpadding;	/* spacing within cells */
      halign_t	halign;		/* table position relative to window */
    } table;
    struct {
      bool		nowrap;	/* suppress word wrap */
      unsigned int	rowspan;/* number of rows spanned by cell */
      unsigned int	colspan;/* number of cols spanned by cell */
      halign_t		halign;	/* horizontal alignment in cells */
      valign_t		valign;	/* vertical alignment in cells */
      Pixels		width;	/* suggested width for cell */
      Pixels		height;	/* suggested height for cell */
    } cell;
  } x;
} Attrib;

typedef struct _html {
  html_t	version;
  union {
    struct {
      void	(*start_html)(String);
      void	(*end_html)(void);
      /**/
      void	(*h1)(String);
      void	(*h2)(String);
      void	(*h3)(String);
      void	(*h4)(String);
      void	(*h5)(String);
      void	(*h6)(String);
      /**/
      struct _block {
	void	(*begin)(void);
	void	(*end)(void);
      } block;
      struct _block	p;
      struct _block	pre;
      struct _block	address;
      /**/
      void	(*strong)(String);
      /**/
      struct {
	void	(*href)(String, String);
	void	(*name)(String, String);
      } a;
      /**/
      void	(*br)(void);
      /**/
      void	(*hr)(void);
      /**/
      void	(*img)(String, String, align_t);
      /**/
      struct {
	void	(*start)(String, method_t);
	void	(*end)(void);
      } form;
      struct {
	void	(*text)(String, String, unsigned int, unsigned int);
	void	(*password)(String, String, unsigned int, unsigned int);
	void	(*checkbox)(String, String, bool);
	void	(*radio)(String, String, bool);
	void	(*image)(String, String);
	void	(*hidden)(String, String);
	void	(*submit)(String, String);
	void	(*reset)(String, String);
      } input;
      struct {
	void	(*start)(String, unsigned int, bool);
	void	(*end)(void);
	void	(*option)(String, bool);
      } select;
      struct {
	void	(*start)(String, unsigned int, unsigned int);
	void	(*end)(void);
      } textarea;
      /**/
      void	(*print)(String);
      int	(*printf)(const char *, ...);
      /**/
      void	(*done)(void);
    } HTML20;

    struct {
      void	(*start_html)(String);
      void	(*end_html)(void);
      /**/
      struct {
	void	(*start)(Attrib *);
	void	(*end)(void);
	void	(*clear)(Attrib *);
      } body;
      /**/
      void	(*h1)(String, halign_t);
      void	(*h2)(String, halign_t);
      void	(*h3)(String, halign_t);
      void	(*h4)(String, halign_t);
      void	(*h5)(String, halign_t);
      void	(*h6)(String, halign_t);
      /**/
      struct _block1 {
	void	(*begin)(void);
	void	(*end)(void);
      } address, pre, center;
      /**/
      struct _block2 {
	void	(*start)(halign_t);
	void	(*end)(void);
      } p, div;
      /**/
      struct _list {
	void	(*start)(Attrib *);
	void	(*end)(void);
	void	(*clear)(Attrib *);
      } ul, ol;
      void	(*li)(String);
      /**/
      struct {
	void	(*href)(String, String);
	void	(*name)(String, String);
      } a;
      /**/
      void	(*strong)(String);
      void	(*font)(String, String);
      /**/
      void	(*br)(void);
      /**/
      struct {
	void	(*start)(Attrib *);
	void	(*clear)(Attrib *);
      } hr;
      /**/
      void	(*img)(String, String, Pixels, Pixels, Pixels, align_t);
      /**/
      struct {
	void	(*start)(String, method_t);
	void	(*end)(void);
      } form;
      struct {
	void	(*start)(String, method_t);
	void	(*end)(void);
      } multipart_form;
      struct {
	void	(*text)(String, String, unsigned int, unsigned int);
	void	(*password)(String, String, unsigned int, unsigned int);
	void	(*checkbox)(String, String, bool);
	void	(*radio)(String, String, bool);
	void	(*submit)(String, String);
	void	(*reset)(String, String);
	void	(*hidden)(String, String);
	void	(*image)(String, String, align_t);
	void	(*file)(String, String, unsigned int, unsigned int);
      } input;
      struct {
	void	(*start)(String, unsigned int, bool);
	void	(*end)(void);
	void	(*option)(String, bool);
      } select;
      struct _textarea {
	void	(*start)(String, unsigned int, unsigned int);
	void	(*end)(void);
      } textarea;
      /**/
      struct {
	void	(*start)(Attrib *);
	void	(*end)(void);
	void	(*clear)(Attrib *);
	void	(*caption)(String, calign_t);
      } table;
      struct {
	void	(*start)(halign_t, valign_t);
	void	(*end)(void);
      } tr;
      struct {
	void	(*start)(Attrib *);
	void	(*end)(void);
	void	(*clear)(Attrib *);
      } th, td;
      /**/
      void	(*print)(String);
      int	(*printf)(const char *, ...);
      /**/
      void	(*done)(void);
    } HTML32;
  } ver;
} HTML;

/* useful macros */
#define	html2	ver.HTML20
#define	html3	ver.HTML32

#define	Body		body.start(NULL)
#define	H1(x)		h1((x),HALIGN_NONE)
#define	H2(x)		h2((x),HALIGN_NONE)
#define	H3(x)		h3((x),HALIGN_NONE)
#define	H4(x)		h4((x),HALIGN_NONE)
#define	H5(x)		h5((x),HALIGN_NONE)
#define	H6(x)		h6((x),HALIGN_NONE)
#define	P		p.start(HALIGN_NONE)
#define	Div		div.start(HALIGN_NONE)
#define	UL		ul.start(NULL)
#define	OL		ol.start(NULL)
#define	Hr		ol.start(NULL)
#define	Table		table.start(NULL)
#define	Caption(x)	table.caption((x),CALIGN_NONE)
#define	Tr		tr.start(HALIGN_NONE,VALIGN_NONE)
#define	Th		th.start(NULL)
#define	Td		td.start(NULL)

/*****************************************************************************/

extern HTML *	newHTML(html_t);

#endif	/* _BASILISK_H */
