/*****************************************************************************
 *
 * FILE:	html20.c
 * DESCRIPTION:	Basilisk: HTML2.0 printer
 * DATE:	Fri, Mar  8 2002
 * UPDATE:	Sun, Aug 19 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2002-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2002-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: html20.c,v 1.4 2007/08/19 16:48:36 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include "basilisk.h"
#include "functions.h"

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
#define	html	ver.HTML20

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static void	html20Done(void);
static void	htmlStart(const String);
static void	htmlEnd(void);
static void	h1(const String);
static void	h2(const String);
static void	h3(const String);
static void	h4(const String);
static void	h5(const String);
static void	h6(const String);
static void	headings(int, const String);
static void	pBegin(void);
static void	pEnd(void);
static void	preBegin(void);
static void	preEnd(void);
static void	addressBegin(void);
static void	addressEnd(void);
static void	block_begin(const String);
static void	block_end(const String);
static void	strong(const String);
static void	aHref(const String, const String);
static void	aName(const String, const String);
static void	br(void);
static void	hr(void);
static void	img(const String, const String, valign_t);
static void	formStart(const String, method_t);
static void	formEnd(void);
static void	inputText(const String, const String,
			  unsigned int, unsigned int);
static void	inputPassword(const String, const String,
			      unsigned int, unsigned int);
static void	inputCheckbox(const String, const String, bool);
static void	inputRadio(const String, const String, bool);
static void	inputImage(const String, const String);
static void	inputHidden(const String, const String);
static void	inputSubmit(const String, const String);
static void	inputReset(const String, const String);
static void	selectStart(const String, unsigned int, bool);
static void	selectEnd(void);
static void	selectOption(const String, bool);
static void	textareaStart(const String, unsigned int, unsigned int);
static void	textareaEnd(void);
static void	Print(String);

/*****************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/
static HTML *	html20 = NULL;	/* html20 object */

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * FUNCTION NAME:	newHTML20
 * DESCRIPTION:		create new HTML20 object
 * ARGUMENTS:		none
 * RETURN VALUE:	Successful: HTML20 object, otherwise: NULL
 */
HTML *
newHTML20(void)
{
  html20 = (HTML *)calloc(1, sizeof(HTML));
  if (html20 == NULL) {
    return NULL;
  }

  html20->html.start_html	= (void *)(htmlStart);
  html20->html.end_html		= (void *)(htmlEnd);
  /**/
  html20->html.h1		= (void *)(h1);
  html20->html.h2		= (void *)(h2);
  html20->html.h3		= (void *)(h3);
  html20->html.h4		= (void *)(h4);
  html20->html.h5		= (void *)(h5);
  html20->html.h6		= (void *)(h6);
  /**/
  html20->html.p.begin		= (void *)(pBegin);
  html20->html.p.end		= (void *)(pEnd);
  html20->html.pre.begin	= (void *)(preBegin);
  html20->html.pre.end		= (void *)(preEnd);
  html20->html.address.begin	= (void *)(addressBegin);
  html20->html.address.end	= (void *)(addressEnd);
  /**/
  html20->html.strong		= (void *)(strong);
  /**/
  html20->html.a.href		= (void *)(aHref);
  html20->html.a.name		= (void *)(aName);
  /**/
  html20->html.br		= (void *)(br);
  /**/
  html20->html.hr		= (void *)(hr);
  /**/
  html20->html.img		= (void *)(img);
  /**/
  html20->html.form.start	= (void *)(formStart);
  html20->html.form.end	= (void *)(formEnd);
  html20->html.input.text	= (void *)(inputText);
  html20->html.input.password	= (void *)(inputPassword);
  html20->html.input.checkbox	= (void *)(inputCheckbox);
  html20->html.input.radio	= (void *)(inputRadio);
  html20->html.input.image	= (void *)(inputImage);
  html20->html.input.hidden	= (void *)(inputHidden);
  html20->html.input.submit	= (void *)(inputSubmit);
  html20->html.input.reset	= (void *)(inputReset);
  html20->html.select.start	= (void *)(selectStart);
  html20->html.select.end	= (void *)(selectEnd);
  html20->html.select.option	= (void *)(selectOption);
  html20->html.textarea.start	= (void *)(textareaStart);
  html20->html.textarea.end	= (void *)(textareaEnd);
  /**/
  html20->html.print		= (void *)(Print);
  html20->html.printf		= (int *)(printf);
  /**/
  html20->html.done		= (void *)(html20Done);

  return html20;
}

static void
html20Done(void)
{
  if (html20) {
    free(html20);
    html20 = NULL;
  }
}

/*****************************************************************************/

static void
htmlStart(title)
	const String	title;
{
  fputs("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n"
	"<html>\n"
	"<head><title>", stdout);
  fprintf(stdout, "%s", title == NULL ? "No title" : title);
  fputs("</title></head>\n<body>\n", stdout);
}

static void
htmlEnd(void)
{
  fputs("</body>\n</html>\n", stdout);
}

/**/
#if	0
static void
h1(text)
	const String	text;
{
  headings(1, text);
}

static void
h2(text)
	const String	text;
{
  headings(2, text);
}

static void
h3(text)
	const String	text;
{
  headings(3, text);
}

static void
h4(text)
	const String	text;
{
  headings(4, text);
}

static void
h5(text)
	const String	text;
{
  headings(5, text);
}

static void
h6(text)
	const String	text;
{
  headings(6, text);
}
#endif
static void h1(const String text) { headings(1, text); }
static void h2(const String text) { headings(2, text); }
static void h3(const String text) { headings(3, text); }
static void h4(const String text) { headings(4, text); }
static void h5(const String text) { headings(5, text); }
static void h6(const String text) { headings(6, text); }

static void
headings(n, text)
	int	n;
	String	text;
{
  fprintf(stdout, "<h%d>%s</h%d>\n", n, text, n);
}

/**/

static void pBegin(void)	{ block_begin("p"); }
static void pEnd(void)		{ block_end("p"); }
static void preBegin(void)	{ block_begin("pre"); }
static void preEnd(void)	{ block_end("pre"); }
static void addressBegin(void)	{ block_begin("address"); }
static void addressEnd(void)	{ block_end("address"); }

static void
block_begin(tag)
	const String	tag;
{
  fprintf(stdout, "<%s>", tag);
}

static void
block_end(tag)
	const String	tag;
{
  fprintf(stdout, "</%s>\n", tag);
}

/**/

static void
strong(text)
	const String	text;
{
  fprintf(stdout, "<strong>%s</strong>", text);
}

/**/

static void
aHref(uri, text)
	const String	uri;
	const String	text;
{
  fprintf(stdout, "<a href=\"%s\">%s</a>", uri, text);
}

static void
aName(name, text)
	const String	name;
	const String	text;
{
  fprintf(stdout, "<a name=\"%s\">%s</a>", name, text);
}

/**/

static void br(void)	{ fputs("<br>\n", stdout); }
static void hr(void)	{ fputs("<hr>\n", stdout); }

/**/

static void
img(src, alt, valign)
	const String	src;
	const String	alt;
	valign_t	valign;
{
  fprintf(stdout, "<img src=\"%s\" alt=\"%s\"", src, alt);
  switch (valign) {
    case VALIGN_TOP:
      fputs(" align=\"top\"", stdout);
      break;
    case VALIGN_MIDDLE:
      fputs(" align=\"middle\"", stdout);
      break;
    case VALIGN_BOTTOM:
      fputs(" align=\"bottom\"", stdout);
      break;
    default:
      break;
  }
  fputs(">\n", stdout);
}

/**/

static void
formStart(action, method)
	const String	action;
	method_t	method;
{
  fprintf(stdout, "<form action=\"%s\" method=", action);
  switch (method) {
    case METHOD_POST:
      fputs("\"post\"", stdout);
      break;
    case METHOD_GET:
    default:
      fputs("\"get\"", stdout);
      break;
  }
  fputs(" enctype=\"application/x-www-form-urlencoded\">\n", stdout);
}

static void
formEnd(void)
{
  fputs("</form>\n", stdout);
}

static void
inputText(name, value, size, maxlength)
	const String	name;
	const String	value;
	unsigned int	size;
	unsigned int	maxlength;
{
  fprintf(stdout, "<input type=\"text\" name=\"%s\" value=\"%s\" "
		  "size=\"%u\" maxlength=\"%u\">",
		  name, value, size, maxlength);
}

static void
inputPassword(name, value, size, maxlength)
	const String	name;
	const String	value;
	unsigned int	size;
	unsigned int	maxlength;
{
  fprintf(stdout, "<input type=\"password\" name=\"%s\" value=\"%s\" "
		  "size=\"%u\" maxlength=\"%u\">",
		  name, value, size, maxlength);
}

static void
inputCheckbox(name, value, checked)
	const String	name;
	const String	value;
	bool		checked;
{
  fprintf(stdout, "<input type=\"checkbox\" name=\"%s\" value=\"%s\"%s>",
		  name, value, checked ? " checked" : "");
}

static void
inputRadio(name, value, checked)
	const String	name;
	const String	value;
	bool		checked;
{
  fprintf(stdout, "<input type=\"radio\" name=\"%s\" value=\"%s\"%s>",
		  name, value, checked ? " checked" : "");
}

static void
inputImage(name, src)
	const String	name;
	const String	src;
{
  fprintf(stdout, "<input type=\"image\" name=\"%s\" src=\"%s\">", name, src);
}

static void
inputHidden(name, value)
	const String	name;
	const String	value;
{
  fprintf(stdout,
	  "<input type=\"hidden\" name=\"%s\" value=\"%s\">", name, value);
}

static void
inputSubmit(name, value)
	const String	name;
	const String	value;
{
  fprintf(stdout,
	  "<input type=\"submit\" name=\"%s\" value=\"%s\">", name, value);
}

static void
inputReset(name, value)
	const String	name;
	const String	value;
{
  fprintf(stdout,
	  "<input type=\"reset\" name=\"%s\" value=\"%s\">", name, value);
}

static void
selectStart(name, size, multiple)
	const String	name;
	unsigned int	size;
	bool		multiple;
{
  fprintf(stdout, "<select name=\"%s\"", name);
  if (size > 0) {
    fprintf(stdout, " size=\"%u\"", size);
  }
  if (multiple) {
    fputs(" multiple>\n", stdout);
  }
  fputs(">\n", stdout);
}

static void
selectEnd(void)
{
  fputs("</select>\n", stdout);
}

static void
selectOption(value, selected)
	const String	value;
	bool		selected;
{
  fprintf(stdout, "<option value=\"%s\"", value);
  if (selected) {
    fputs(" selected>\n", stdout);
  }
  fputs(">\n", stdout);
}

static void
textareaStart(name, rows, cols)
	const String	name;
	unsigned int	rows;
	unsigned int	cols;
{
  fprintf(stdout,
	  "<textarea name=\"%s\" rows=\"%u\" cols=\"%u\">\n", name, rows, cols);
}

static void
textareaEnd(void)
{
  fputs("</textarea>\n", stdout);
}

static void
Print(text)
	const String	text;
{
  if (text != NULL) {
    fputs(text, stdout);
  }
}
